#! /usr/bin/python
#Author Mika Spara
#This work is licensed under the Creative Commons #Attribution-Noncommercial-Share Alike 4.0 Unported License. To view a copy of #this license, visit http://creativecommons.org/licenses/by-nc-sa/4.0/

msg_version = "Version 180909"
msg_copyright = "Copyright: Mika Spara, 2008-2018. mika.spara@hotmail.com"
msg_license = "This work is licensed under the Creative Commons Attribution-Noncommercial-Share Alike 4.0 Unported License."


import ConfigParser as confpar
import pygame as p
import random as r
import math as m
import time
import sys
from pygame.locals import *

class Score(p.sprite.Sprite):
    def __init__(self):
        p.sprite.Sprite.__init__(self)
        self.image = p.Surface([200, 50], SRCALPHA, 32).convert_alpha()
        self.image.fill((0, 0, 0, 0))
        self.rect = self.image.get_rect()
        self.rect.topleft = [15, 15]
        self.font = p.font.Font(fontface, 48)

class Impact(p.sprite.Sprite):
    def __init__(self, type):
        p.sprite.Sprite.__init__(self)
        if type==0 :
            self.image_orig =  p.image.load('data/emp.png').convert_alpha()
        else:
            self.image_orig =  p.image.load('data/explosion.png').convert_alpha()
        self.image = self.image_orig
        self.rect = self.image.get_rect()
    def new_place(self, x ,y):
        self.rect.center = (x, y)
    def new_size(self, size):
        location = self.rect.center
        imp_width = size
        self.image = p.transform.scale(self.image_orig, (imp_width, imp_width))
        self.rect = self.image.get_rect()
        self.rect.center = location

class Missile(p.sprite.Sprite):
    def __init__(self):
        p.sprite.Sprite.__init__(self)
        self.image =  p.image.load('data/missile.png').convert_alpha()
        self.rect = self.image.get_rect()
    def new_place(self, x, y):
        self.rect.center = (x, y)

class Puff(p.sprite.Sprite):
    def __init__(self, size):
        puff_width = int(scale * size)
        p.sprite.Sprite.__init__(self)
        self.image =  p.transform.scale(p.image.load('data/smoke.png').convert_alpha(),(puff_width, puff_width))
        self.rect = self.image.get_rect()
    def new_place(self, x, y):
        self.rect.center = (x, y)

#Sprite class for the life count
class Life(p.sprite.Sprite):
    def __init__(self, side):
        p.sprite.Sprite.__init__(self)
        self.image = p.image.load('data/ufo_yellow.png').convert_alpha()
        if side:
            self.image = p.image.load('data/ufo_red.png').convert_alpha()
        self.rect = self.image.get_rect()
    def new_place(self, x, y):
        self.rect.center = (x, y)

#Sprite class for players
class Player(p.sprite.Sprite):
    def __init__(self, side):
        p.sprite.Sprite.__init__(self)
        if scale < 1.25:
            self.image = p.image.load('data/ufo.png')
        elif scale < 1.5:
            self.image = p.image.load('data/ufo2.png')
        else:
            self.image = p.image.load('data/ufo3.png')
        self.rect = self.image.get_rect()
        self.lives = 0
        self.heading = 0.0
        self.power = 0.0
        self.color = (0, 0, 0)
    def new_place(self, x, y):
        self.rect.center = (x, y)
    def lives_update(self, lives):
        self.lives = lives
    #Method for game start animation.
    def zoom_in(self):
        x0 = self.rect.center[0]
        y0 = self.rect.center[1]
        screen.set_clip((x0 - 20, y0 - 20), (40, 40))
        for ufo_width in range(int(self.rect[2])):
            clock.tick(scale * 30)
            scaled=p.transform.scale(self.image, (ufo_width, ufo_width))
            screen.blit(scaled, (x0 - m.floor(ufo_width / 2.0), m.floor(y0 - ufo_width / 2.0)))
            p.display.update((x0 - 20, y0 - 20), (40, 40))
        screen.blit(bg, (0, 0))
        playersprites.draw(screen)
        p.display.update((x0 - 20, y0 - 20), (40, 40))
    #Move player out of sight.
    def hide(self):
        self.rect.center = (-30, 0)

#Draw the aiming sight for player in turn.
def draw_aim(rotate_diff, velocity_diff):
    rect_clip = int(round(scale * 70))
    aim_color = (0, 200, 0)
    player_x = player[turn].rect.center[0]
    player_y = player[turn].rect.center[1]
    aim_d = player[turn].power - 120
    aim_rad = m.radians(player[turn].heading)
    tangent_d = int(round(scale * 65))
    tangent_x = player_x + tangent_d*m.cos(aim_rad)
    tangent_y = player_y + tangent_d*m.sin(aim_rad)
    screen.set_clip((player_x - rect_clip, player_y - rect_clip), (2 * rect_clip, 2 * rect_clip))
    #screen.blit(surf_stars, (0, 0))
    screen.blit(bg_temp, (0, 0))
    playersprites.draw(screen)
    p.draw.line(screen, aim_color, (player_x, player_y), (round(tangent_x), round(tangent_y)), 1)
    p.draw.circle(screen, aim_color, (player_x, player_y), tangent_d + 1, 1)
    #Tick marks.
    for i in [0, 0.5, 1, 1.5, 2]:
        start_x = player_x + (tangent_d-10) * m.cos(i * m.pi)
        start_y = player_y + (tangent_d-10) * m.sin(i * m.pi)
        end_x = player_x + (tangent_d) * m.cos(i * m.pi)
        end_y = player_y + (tangent_d) * m.sin(i * m.pi)
        p.draw.line(screen, aim_color, (round(start_x), round(start_y)),(round(end_x), round(end_y)), 1)
    for i in [0.125, 0.25, 0.375, 0.625, 0.75, 0.875, 1.125, 1.25, 1.375, 1.625, 1.75, 1.875]:
        start_x=player_x + (tangent_d-5) * m.cos(i * m.pi)
        start_y=player_y + (tangent_d-5) * m.sin(i * m.pi)
        end_x=player_x + (tangent_d) * m.cos(i * m.pi)
        end_y=player_y + (tangent_d) * m.sin(i * m.pi)
        p.draw.line(screen, aim_color, (round(start_x), round(start_y)), (round(end_x), round(end_y)), 1)
    #Power meter.
    for i in range(60, int(player[turn].power) + 1, 4):
        aim_rad = m.radians(player[turn].heading - 1)
        aim_d = int(round(scale * (5.0 / 4.0 * i - 65)))
        p.draw.arc(screen, aim_color, ((player_x - aim_d, player_y - aim_d), (2 * aim_d, 2 * aim_d)), -aim_rad - m.pi / 8, -aim_rad + m.pi / 8, 1)
    font = p.font.Font(fontface, 12)
    rotate_str = str(rotate_diff)
    if (rotate_diff > 0):
        rotate_str = "+" + rotate_str
    rotate_text = font.render(rotate_str, 1, (0, 200, 0))
    screen.blit(rotate_text, (player_x - tangent_d, player_y - tangent_d))
    velocity_str = str(velocity_diff)
    if (velocity_diff > 0):
        velocity_str = "+" + velocity_str
    velocity_text = font.render(velocity_str, 1, (0, 200, 0))
    screen.blit(velocity_text, (player_x - tangent_d, player_y - tangent_d+10))
    p.display.update((player_x - rect_clip, player_y - rect_clip), (2 * rect_clip, 2 * rect_clip))

#Focus animation
def focus():
    global bg_temp
    screen.set_clip()
    steps = 50
    if turn:
        x_start=0
    else:
        x_start=width
    y_start=0
    player_x = player[turn].rect.center[0]
    player_y = player[turn].rect.center[1]
    y_step = (player_y - y_start) / steps
    x_step = (player_x - x_start) / steps

    for i in (range(0, steps)):
        clock.tick(60)
        screen.blit(bg, (0, 0))
        playersprites.draw(screen)
        scoresprites.draw(screen)
        p.draw.line(screen, (0, 200, 0), (0, y_start + i * y_step), (width, y_start + i * y_step), 1)
        p.draw.line(screen, (0, 200, 0), (x_start + i * x_step, 0), (x_start + i * x_step, height), 1)
        p.display.flip()

    bg.set_clip()
    bg_temp.set_clip()
    bg_temp = bg.copy()
    bg_temp.blit(surf_highlight[turn], (0,0))
    screen.blit(bg_temp, (0, 0))
    playersprites.draw(screen)
    scoresprites.draw(screen)
    p.display.flip()
    p.event.clear()

#Create the background with planets and starfield.
def createbg(count, density):
    bg.set_clip()
    screen.set_clip()
    surf_stars.set_clip()
    surf_planets.set_clip()
    surf_planets_edge.set_clip()
    surf_planets_hole.set_clip()
    surf_planets.fill((0, 0, 0, 0))
    surf_planets_edge.fill((0, 0, 0, 0))
    surf_planets_hole.fill((0, 0, 0, 0))
    red = p.image.load('data/red_alt.png').convert_alpha()
    green = p.image.load('data/green_alt.png').convert_alpha()
    blue = p.image.load('data/blue_alt.png').convert_alpha()
    yellow = p.image.load('data/yellow_alt.png').convert_alpha()
    ready = 0
    #Create a suitable set of planets to play with.
    while ready == 0:
        ready = 1
        #Init planet array.
        for i in range(len(planets)):
            del planets[0]
        i = 0
        k = 0
        x = [0.0, 0.0]
        y = [0.0, 0.0]
        #Pick place for a planet, not a black planet, between players.
        x[0] = float(player[0].rect.center[0] - x_off)/scale
        y[0] = float(player[0].rect.center[1] - y_off)/scale
        x[1] = float(player[1].rect.center[0] - x_off)/scale
        y[1] = float(player[1].rect.center[1] - y_off)/scale
        a = int(round(200 * (x[1] - x[0]) / m.sqrt((x[1] - x[0]) ** 2 + (y[1] - y[0]) ** 2)))
        radius = r.randint(30, 48)
        x0 = float(r.randint(int(x[0]) + a + radius, int(x[1]) - a - radius))
        y0 = (y[1] - y[0]) / (x[1] - x[0]) * (x0 - x[0]) + y[0]
        d =  (y[1] - y[0]) / m.fabs(y[1] - y[0]) * float(radius) / 2
        dy = d*(x[0] - x[1]) / m.sqrt((y[1] -y[0])**2 + (x[0] - x[1])**2)
        dx = m.sqrt(d**2 - dy**2)
        direction = r.choice([-1 ,1])
        x3 = x0 + direction * dx
        y3 = y0 + direction * dy
        planets.append([int(x3), int(y3), radius, r.randint(1, 4)])
        #Pick places for the rest of the planets.
        while i < count - 1 and ready == 1:
            k = k + 1
            #Randomize x-coord, y-coord, radius and density.
            planet = [r.randint(200, 600), r.randint(100, 500), r.randint(30, 50), r.randint(1, 4)]
            ok = 1
            #Make sure that planets have are at least 100 units of space between them.
            for j in range(len(planets)):
                if m.sqrt((planet[0] - planets[j][0])**2 + (planet[1] - planets[j][1])**2) < (100 + planet[2] + planets[j][2]):
                    ok = 0
            #Make sure that players and planets have at least 200 units of space between them.
            if ok:
                for j in [0, 1]:
                    if m.sqrt((planet[0] - x[j])**2 + (planet[1] - y[j])**2) < (planet[2] + 200):
                        ok = 0
            if ok:
                planets.append(planet)
                i = i + 1
            #If it takes too long to find a suitable set, start over again.
            if k == 100:
                ready = 0
        sum = 0
        if ready:
            for j in range(len(planets)):
                #Create a black planet.
                if planets[j][2] >= 49:
                    planets[j][2] = 10
                    planets[j][3] = 500
                sum=sum + (float(0.5 * (planets[j][3] + 0.5) * pow(planets[j][2], 3)))
            #Make sure that the overall mass of the system is between fixed values.
            if sum < 500000 or sum > 600000 :
                ready = 0
    #Create a starfield.
    starfield(density)
    #Scale planets to screen resolution and put planets to the background surface.
    for i in range(len(planets)):
        planets[i][0] = int(scale * planets[i][0] + x_off)
        planets[i][1] = int(scale * planets[i][1] + y_off)
        planets[i][2] = int(scale * planets[i][2])
        if planets[i][2]== int(scale * 10):
            p.draw.circle(surf_planets, (0, 0, 0), (planets[i][0], planets[i][1]), planets[i][2], 0)
        else:
            if planets[i][3]==4:
                color=red
            elif planets[i][3]==3:
                color=blue
            elif planets[i][3]==2:
                color=green
            else:
                color=yellow
            scaled=p.transform.scale(p.transform.flip(color, r.randint(0,1), r.randint(0,1)),[(planets[i][2]+1)*2, (planets[i][2]+1)*2])
            #Show planets on the background
            if show_planets == 1:
                p.draw.circle(surf_stars, (0, 0, 0, 0), (planets[i][0], planets[i][1]), planets[i][2]-1, 0)
            surf_planets.blit(scaled, [planets[i][0]-planets[i][2]-1,planets[i][1]-planets[i][2]-1])
    bg.blit(surf_planets, (0, 0))
    bg.blit(surf_stars, (0, 0))

#Create a starfield
def starfield(density):
    star_count = width * height * density / (800 * 600)
    surf_stars.set_clip()
    surf_stars.fill((0, 0, 0))
    for i in range(star_count):
        star=[r.randint(1,width),r.randint(1,height)]
        shade = r.randint(1,255)
        color = [[shade, shade, shade], [0, 0, shade / 2], [shade / 4, shade / 4, 0], [shade / 3, 0 ,0]]
        #green [0, shade / 4 ,0]
        size = r.randint(1, 3)
        #Create some variation to the boring shades of gray.
        variate = r.randint(1, 50)
        if variate < 3:
            p.draw.line(surf_stars, color[1],[star[0]-size, star[1]], [star[0]+size, star[1]])
            p.draw.line(surf_stars, color[1],[star[0], star[1]-size], [star[0], star[1]+size])
        elif variate < 5:
            p.draw.line(surf_stars,color[2],[star[0]-size, star[1]], [star[0]+size, star[1]])
            p.draw.line(surf_stars,color[2],[star[0], star[1]-size], [star[0], star[1]+size])
        elif variate < 7:
            p.draw.line(surf_stars,color[3],[star[0]-size, star[1]], [star[0]+size, star[1]])
            p.draw.line(surf_stars,color[3],[star[0], star[1]-size], [star[0], star[1]+size])
        p.draw.line(surf_stars,color[0],star,star)

#Initialize and pick places for players.
def initplayers():
    i = 1
    while i:
        i = 0
        #X-coord for player 1 and player 2.
        x = [100.0, 700.0]
        #Y-coord for players 1 and player 2.
        y = [float(r.randint(100,500)), float(r.randint(100,500))]
        #Make sure that players have enough space between them.
        if (x[0] - x[1])**2 + (y[0] - y[1])**2 < 450000:
            i = 1
    player[0].new_place(int(round(x_off + scale * x[0])), int(round(y_off + scale * y[0])))
    player[0].heading = 360
    player[0].power = 80.0
    player[0].color = (128, 128, 0)
    player[0].hcolor = (255, 255, 0)
    player[1].new_place(int(round(x_off + scale * x[1])), int(round(y_off + scale * y[1])))
    player[1].heading =180
    player[1].power = 80.0
    player[1].color = (128,0,0)
    player[1].hcolor = (255,0,0)

#Change heading.
def rotate(player_number, direction):
    if direction == 0:
        new_heading = player[player_number].heading - 0.5
        if new_heading < 0.5:
            new_heading = 360
    else:
        new_heading = player[player_number].heading + 0.5
        if new_heading > 360.5:
            new_heading = 1
    player[player_number].heading = new_heading

#Change power.
def velocity(player_number,direction):
    if direction == 0:
        new_power = player[player_number].power - 4
        if new_power < 60:
            new_power = 60
    else:
        new_power = player[player_number].power + 4
        if new_power > 100:
            new_power = 100
    player[player_number].power = new_power

#Calculate the next point.
def trace(x0, y0, v0x, v0y):
    x0 = (x0 - x_off) / scale
    y0 = (y0 - y_off) / scale
    t = 0.005
    a_x = 0.0
    a_y = 0.0
    #Calculate gravitational acceleration components.
    for i in range(len(planets)):
        planet_r = planets[i][2] / scale
        density = (0.5 * planets[i][3] + 0.5) * 20
        planet_x = float(planets[i][0] - x_off) / scale
        planet_y = float(planets[i][1] - y_off) / scale
        dx = planet_x - x0
        dy = planet_y - y0
        dist = m.sqrt(dx**2 + dy**2)
        if (dist < planet_r):
            planet_r = dist
        planet_m = float(density * pow(planet_r, 3))
        g = planet_m/(dist**2)
        g_x = g * dx/dist
        g_y = g * dy/dist
        a_x = a_x + g_x
        a_y = a_y + g_y
    #Calculate next trace point
    x1 = x0 + v0x * t + 0.5 * a_x * t**2
    y1 = y0 + v0y * t + 0.5 * a_y * t**2
    #Calculate next velocity.
    v1x = v0x + a_x * t
    v1y = v0y + a_y * t
    return (x_off + scale * x1, y_off + scale * y1, v1x, v1y, a_x, a_y)

#Check for area exit.
def exit_area(x, y):
    if (x > 1.1 * width or y > 1.1 * height or x < -0.1 * width or y < -0.1 * height):
        snd = p.mixer.Sound("data/gone.wav")
        wait = snd.get_length()
        snd.play()
        time.sleep(wait)
        return 1
    else:
        return -1

#Check if the torpedo hits a planet.
def planet_hit_torpedo(x, y):
    if ((x > 1 and x < width - 1) and (y > 1 and y < height - 1)):
        if surf_planets.get_at((int(round(x)), int(round(y))))[3] == 255:
            return (1, x, y)
    return (-1, x, y)

#Check if player hits a planet.
def planet_hit_player(x, y, d):
    if ((x > 21 and x < width - 21) and (y > 21 and y < height - 21)):
        for i in range(len(planets)):
            planet_x = float(planets[i][0])
            planet_y = float(planets[i][1])
            planet_r = float(planets[i][2])
            if ((x - planet_x)**2 + (y - planet_y)**2) < (planet_r + d)**2:
                for j in range(60):
                    edge_x = int(round(x + d * m.cos(float(j)/30.0 * m.pi)))
                    edge_y = int(round(y + d * m.sin(float(j)/30.0 * m.pi)))
                    if surf_planets.get_at((edge_x, edge_y))[3] == 255:
                        return (1, edge_x, edge_y)
    return (-1, x, y)

#Check if the torpedo hits a player.
def player_hit_torpedo(x, y):
    for i in [0, 1]:
        player_x = player[i].rect.center[0]
        player_y = player[i].rect.center[1]
        if (x - player_x)**2 + (y - player_y)**2 < (scale * 10)**2:
            return (i, x, y)
    return (-1, x, y)

#Check if player collides with player.
def player_hit_player(x, y, player_number):
    player_x = player[player_number].rect.center[0]
    player_y = player[player_number].rect.center[1]
    if (x - player_x)**2 + (y - player_y)**2 < (scale * 20)**2:
        return (1, (x + player_x) / 2, (y + player_y) / 2)
    return (-1, x, y)

#Trace torpedo
def trace_torpedo(player_number):
    if (draw_forces == 1):
        anim_speed = 250
    else:
        anim_speed = 300
    surf_highlight[player_number] = p.Surface(screen.get_size(), SRCALPHA, 32).convert_alpha()
    snd_fire.play()
    missilesprite = p.sprite.OrderedUpdates(missile)
    staticsprites = p.sprite.OrderedUpdates(player, score)
    x = player[player_number].rect.center[0]
    y = player[player_number].rect.center[1]
    initial_heading = player[player_number].heading
    initial_power = player[player_number].power
    color = player[player_number].color
    hcolor = player[player_number].hcolor
    #Clean aiming sight from screen.
    screen.set_clip()
    screen.blit(bg, (0,0))
    playersprites.draw(screen)
    scoresprites.draw(screen)
    p.display.flip()
    p.display.update((int(round(x - scale * 70)), int(round(y - scale * 70))), (int(round(scale * 140)), int(round(scale * 140))))
    x0 = x + (scale * 10) * m.cos(m.radians(initial_heading))
    y0 = y + (scale * 10) * m.sin(m.radians(initial_heading))
    v0x = (0.6 * initial_power + 130) * m.cos(m.radians(initial_heading))
    v0y = (0.6 * initial_power + 130) * m.sin(m.radians(initial_heading))
    distance = 0
    prev_rect = Rect(missile.rect)
    bg_temp = bg.copy()
    #Trace missile and draw lines.
    count = 0
    while 1:
        clock.tick(anim_speed)
        next_point = trace(x0, y0, v0x, v0y)
        x1 = next_point[0]
        y1 = next_point[1]
        x_min = min(x0, x1)
        y_min = min(y0, y1)
        distance = distance + m.sqrt((x1 - x0)**2 + (y1- y0)**2)
        missile.new_place(int(round(x1)), int(round(y1)))
        draw_rect = prev_rect.union(missile.rect)
        prev_rect = Rect(missile.rect)
        bg.set_clip(draw_rect)
        surf_stars.set_clip(draw_rect)
        if draw_traces == 1:
            p.draw.line(bg, color, (int(round(x0)), int(round(y0))), (int(round(x1)), int(round(y1))),1)
            p.draw.line(bg_temp, hcolor, (int(round(x0)), int(round(y0))), (int(round(x1)), int(round(y1))),1)
            p.draw.line(surf_stars, color, (int(round(x0)), int(round(y0))), (int(round(x1)), int(round(y1))),1)
            p.draw.line(surf_highlight[player_number], hcolor, (int(round(x0)), int(round(y0))), (int(round(x1)), int(round(y1))),1)
        if draw_forces == 1:
            vector_rect_x = min(int(round(x1)), int(round(x1+next_point[4]/5)));
            vector_rect_y = min(int(round(y1)), int(round(y1+next_point[5]/5)));
            vector_rect_width = m.fabs(next_point[4]/5)
            vector_rect_height = m.fabs(next_point[5]/5)
            vector_rect = Rect(vector_rect_x - 3, vector_rect_y - 3, vector_rect_width + 6, vector_rect_height + 6)
            prev_rect = vector_rect.union(Rect(missile.rect))
            draw_rect = draw_rect.union(vector_rect)
        screen.set_clip(draw_rect)
        screen.blit(bg_temp, (0,0))
        if draw_forces == 1:
            #p.draw.line(screen, (0, 255,0),(int(round(x1)), int(round(y1))), (int(round(x1+next_point[4]/5)), int(round(y1))))
            #p.draw.line(screen, (0, 255,0),(int(round(x1)), int(round(y1))), (int(round(x1)), int(round(y1+next_point[5]/5))))
            p.draw.line(screen, (0, 255,0),(int(round(x1)), int(round(y1))), (int(round(x1+next_point[4]/5)), int(round(y1+next_point[5]/5))))
            count = count + 1
            if (count % 10 == 0):
                p.draw.line(bg_temp, (0, 255,0),(int(round(x1)), int(round(y1))), (int(round(x1+next_point[4]/5)), int(round(y1+next_point[5]/5))))
                p.draw.line(surf_highlight[player_number], (0, 255,0),(int(round(x1)), int(round(y1))), (int(round(x1+next_point[4]/5)), int(round(y1+next_point[5]/5))))
                count = 0
        missilesprite.clear(screen, bg_temp)
        missilesprite.draw(screen)
        staticsprites.draw(screen)
        p.display.update(draw_rect)
        #Exit area?
        if exit_area(x1, y1) == 1:
            return (2, x0, y0)
        #Planet hit?
        result = planet_hit_torpedo(x1, y1)
        if result[0] == 1:
            return (3, result[1], result[2])
        #Player hit?
        result = player_hit_torpedo(x1, y1)
        if result[0] != -1:
            cool_value = distance / (m.sqrt((player[0].rect.center[0] - player[1].rect.center[0])**2 + (player[0].rect.center[1] - player[1].rect.center[1])**2) - scale * 20)
            return (result[0], result[1], result[2], cool_value)
        x0 = x1
        y0 = y1
        v0x = next_point[2]
        v0y = next_point[3]

#Trace player
def trace_player(player_number, impact_x, impact_y):
    screen.set_clip()
    screen.blit(bg, (0,0))
    playersprites.draw(screen)
    scoresprites.draw(screen)
    p.display.flip()
    snd_damage.play()
    x0 = player[player_number].rect.center[0]
    y0 = player[player_number].rect.center[1]
    hit[0].new_place(impact_x, impact_y)
    impactsprites = p.sprite.OrderedUpdates(player, hit[0])
    for i in range(len(trail)):
        trail[i].new_place(x0, y0)
    allsprites = p.sprite.OrderedUpdates(player, trail)
    v0x = 5 * (x0 - impact_x) / scale
    v0y = 5 * (y0 - impact_y) / scale
    expl_calc = 2
    while 1:
        clock.tick(240)
        next_point = trace(x0, y0, v0x, v0y)
        x1 = next_point[0]
        y1 = next_point[1]
        player[player_number].new_place(x1, y1)
        #Draw impact effect
        if expl_calc < 22:
            impactsprites.clear(screen, bg)
            expl_calc = expl_calc + 0.25
            if expl_calc < 12:
                rad = expl_calc
            else:
                rad = 22 - expl_calc
            hit[0].new_size(int(scale * 2* rad))
            p.display.update(impactsprites.draw(screen))
            if expl_calc == 22:
                for i in range(len(trail)):
                    trail[i].new_place(x1, y1)
                trail_step = 10
                trail_count = 0
        #Draw smoke trail
        else:
            allsprites.clear(screen, bg)
            trail_count = trail_count + 1
            if trail_count == trail_step:
                for  i in range(4, 0, -1):
                    trail[i].new_place(trail[i-1].rect.center[0], trail[i-1].rect.center[1])
                trail[0].new_place(x0, y0)
                trail_count = 0
            draw_rect = allsprites.draw(screen)
            scoresprites.draw(screen)
            p.display.update(draw_rect)
        #Exit area?
        if exit_area(x1, y1) == 1:
            return (2, x0, y0)
        #Planet hit?
        result = planet_hit_player(x1, y1, scale * 10)
        if result[0] == 1:
            return (3, result[1], result[2])
        #Player collision?
        result = player_hit_player(x1, y1, (player_number + 1) % 2)
        if result[0] == 1:
            return (4, result[1], result[2])
        x0 = x1
        y0 = y1
        v0x = next_point[2]
        v0y = next_point[3]

#Draw explosion
def explosion(exp_x, exp_y, size, hide=2):
    size = int(round(scale * size))
    screen.set_clip()
    #Check if it's a missile or a player explosion.
    if hide == 2:
        snd_damage.play()
        type = 0
    else:
        snd_explosion.play()
        type = 1
    impactsprites = p.sprite.OrderedUpdates(hit[type])
    hit[type].new_place(exp_x, exp_y)
    #Check for planet hit
    if show_planets == 0 and hide == 2:
        for i in range(len(planets)):
            planet_x = int(planets[i][0])
            planet_y = int(planets[i][1])
            planet_r = int(planets[i][2])
            if ((exp_x - planet_x)**2 + (exp_y - planet_y)**2) < (planet_r +1)**2 and planets[i][3] != 500:
                if planets[i][3]==4:
                    circ_col = (255, 0, 0)
                elif planets[i][3]==3:
                    circ_col = (0, 0, 255)
                elif planets[i][3]==2:
                    circ_col = (0, 255, 0)
                else:
                    circ_col = (255, 255, 0)
                p.draw.circle(screen, circ_col, (planet_x, planet_y), planet_r, 1)
                p.display.update((planet_x - planet_r, planet_y - planet_r),(2 * planet_r, 2 * planet_r))

    #Draw expanding filled circles
    for i in range(size):
        clock.tick(scale * 60)
        hit[type].new_size(2 * i)
        p.display.update(impactsprites.draw(screen))
    #Hide player sprite on player explosion.
    if hide != 2:
        if hide == 3:
            player[0].hide()
            player[1].hide()
        else:
            player[hide].hide()
    #Update planets surface
    if surf_planets.get_at((int(round(exp_x)), int(round(exp_y)))) != (0, 0, 0, 255):
        surf_planets_edge.set_clip((exp_x - size, exp_y - size), (2 * size, 2 * size))
        surf_planets_hole.set_clip((exp_x - size, exp_y - size), (2 * size, 2 * size))
        surf_planets.set_clip((exp_x - size, exp_y - size), (2 * size, 2 * size))
        p.draw.circle(surf_planets_edge, (31, 31, 31),(int(round(exp_x, 0)),int(round(exp_y, 0))),size ,0)
        p.draw.circle(surf_planets_hole,(15, 15, 15) ,(int(round(exp_x, 0)),int(round(exp_y, 0))),size-2,0)
        p.draw.circle(surf_planets, (0, 0, 0, 0),(int(round(exp_x, 0)),int(round(exp_y, 0))),size,0)
    bg.set_clip((exp_x - size, exp_y - size), (2 * size, 2 * size))
    bg.blit(surf_planets, (0, 0))
    bg.blit(surf_planets_edge, (0, 0))
    bg.blit(surf_planets_hole, (0, 0))
    bg.blit(surf_stars, (0, 0))
    screen.blit(bg, (0,0))
    #Draw shrinking filled circles
    scoresprites.draw(screen)
    impactsprites.draw(screen)
    p.display.update((exp_x-size, exp_y-size),(2 * size, 2 * size))
    screen.set_clip()
    for i in range(size):
        clock.tick(scale * 60)
        hit[type].new_size(2 * (size - i))
        p.display.update(impactsprites.draw(screen))
        impactsprites.clear(screen, bg)
    playersprites.draw(screen)
    p.display.flip()

#Print text to surface.
def text_to_surface(surf, msg, x, position, size):
    font = p.font.Font(fontface, size)
    text = font.render(msg, 1, (0, 255, 0))
    if x=="c":
        size = font.size(msg)
        x = width / 2 - size[0]/2
    if position=="c":
        size = font.size(msg)
        position = height / 2 - size[1]/2
    textpos = (x, position)
    surf.blit(text, textpos)

#Game opening screen and game start.
def game_start():
    global turn
    global shots
    #Create a nice background.
    bg.set_clip()
    bg.fill((0, 0, 0))
    starfield(1000)
    bg.blit(surf_stars, (0, 0))
    selected = 0
    draw_menu(selected)
    i = 1
    #Menu loop
    while i:
        clock.tick(10)
        for event in p.event.get():
            if event.type == QUIT:
                sys.exit()
            if event.type == KEYDOWN:
                if event.key == K_SPACE or event.key == K_RIGHT  or event.key == K_LEFT or event.key == K_RETURN:
                    if selected == 0:
                        i = 0
                    elif selected == 1:
                        options()
                    elif selected == 2:
                        sys.exit()
                if event.key == K_UP:
                    if selected > 0:
                        selected = selected - 1
                if event.key == K_DOWN:
                    if selected < 2:
                        selected = selected + 1
                if event.key == K_ESCAPE:
                    sys.exit()
        draw_menu(selected)
        p.display.flip()
    player[0].lives = max_lives
    player[1].lives = max_lives
    turn = r.randint(0, 1)

#Options menu
def options():
    global fullscreen
    global screen
    global max_lives
    global width
    global height
    global draw_traces
    global draw_forces
    global show_planets
    global anim_speed
    selected = 0
    show_mode = p.display.list_modes().index(screen.get_size())
    options_menu(selected, show_mode)
    i = 1
    #Menu loop
    while i:
        clock.tick(10)
        for event in p.event.get():
            if event.type == QUIT:
                sys.exit()
            if event.type == KEYDOWN:
                if event.key == K_SPACE or event.key == K_RIGHT  or event.key == K_LEFT or event.key == K_RETURN:
                    if selected == 0:
                        altered = 0
                        display_modes = p.display.list_modes()
                        valid_modes = []
                        for j in range(len(display_modes)):
                            if display_modes[j][0] >= 800 and display_modes[j][0] > display_modes[j][1]:
                                valid_modes.append(display_modes[j])
                        if event.key == K_RIGHT and show_mode > 0:
                            show_mode = show_mode - 1
                        if event.key == K_LEFT and show_mode < len(valid_modes)-1:
                            show_mode = show_mode + 1
                        if event.key == K_SPACE or event.key == K_RETURN:
                            width = valid_modes[show_mode][0]
                            height = valid_modes[show_mode][1]
                            write_init("width", width)
                            write_init("height", height)
                            resolution_init()
                            bg.set_clip()
                            bg.fill((0, 0, 0))
                            starfield(1000)
                            bg.blit(surf_stars, (0, 0))
                    elif selected == 1:
                        fullscreen = (fullscreen + 1) % 2
                        save_screen = screen.copy()
                        screen = init_display(fullscreen, width, height)
                        screen.blit(save_screen, (0,0))
                        write_init('fullscreen', fullscreen)
                    elif selected == 2:
                        if event.key == K_LEFT and max_lives > 1:
                            max_lives = max_lives - 1
                        elif (event.key == K_RIGHT or event.key == K_SPACE or event.key == K_RETURN)  and max_lives < 10:
                            max_lives = max_lives + 1
                        write_init('max_lives', max_lives)
                    elif selected == 3:
                        draw_traces = (draw_traces + 1) % 2
                        write_init('draw_traces', draw_traces)
                    elif selected == 4:
                        show_planets = (show_planets + 1) % 2
                        write_init('show_planets', show_planets)
                    elif selected == 5:
                        draw_forces = (draw_forces + 1) % 2
                        write_init('draw_forces', draw_forces)
                    elif selected == 6:
                        return
                if event.key == K_UP:
                    if selected > 0:
                        selected = selected - 1
                if event.key == K_DOWN:
                    if selected < 6:
                        selected = selected + 1
                if event.key == K_ESCAPE:
                    return
        options_menu(selected, show_mode)
        p.display.flip()

#Draw options menu
def options_menu(choice, show_mode):
    x_pad = (width - 800)/2
    y_pad = (height - 600)/2
    selector = p.sprite.RenderPlain(player[0])
    screen.set_clip()
    screen.blit(bg, (0,0))
    text_to_surface(screen, "Options", "c" , y_pad + 25, 72)
    msg = "Resolution: %s" % (str(p.display.list_modes()[show_mode]))
    text_to_surface(screen, msg, x_pad + 320, y_pad + 185, 36)
    fs = "Off"
    if fullscreen == 1:
        fs = "On"
    msg = "Fullscreen: %s" % (fs)
    text_to_surface(screen, msg, x_pad + 320, y_pad + 230, 36)
    msg = "Life count: %i" % (max_lives)
    text_to_surface(screen, msg, x_pad + 320, y_pad + 275, 36)
    msg = "Draw traces: %i" % (draw_traces)
    text_to_surface(screen, msg, x_pad + 320, y_pad + 320, 36)
    msg = "Show planets: %i" % (show_planets)
    text_to_surface(screen, msg, x_pad + 320, y_pad + 365, 36)
    msg = "Draw gravity vector: %i" % (draw_forces)
    text_to_surface(screen, msg, x_pad + 320, y_pad + 410, 36)
    msg = "Return"
    text_to_surface(screen, msg, x_pad + 320, y_pad + 455, 36)
    msg = msg_copyright
    text_to_surface(screen, msg, "c" , y_pad + 550, 16)
    msg = msg_license
    text_to_surface(screen, msg, "c" , y_pad + 575, 16)
    player[0].new_place(x_pad + 295, y_pad + 203 + choice * 45)
    selector.draw(screen)
    yellow = p.image.load('data/yellow_alt.png')
    screen.blit(yellow, (x_pad + 50, y_pad + 340))
    green = p.image.load('data/green_alt.png')
    screen.blit(green, (x_pad + 50, y_pad + 265))
    blue = p.image.load('data/blue_alt.png')
    screen.blit(blue, (x_pad + 50, y_pad + 190))
    red = p.image.load('data/red_alt.png')
    screen.blit(red, (x_pad + 50, y_pad + 115))
    p.draw.circle(screen,(0, 0, 0), (x_pad + 145, y_pad + 150), 40, 0)
    p.display.flip()

#Draw main menu
def draw_menu(choice):
    x_pad = (width - 800)/2
    y_pad = (height - 600)/2
    selector = p.sprite.RenderPlain(player[0])
    screen.set_clip()
    screen.blit(bg, (0,0))
    text_to_surface(screen, msg_version, x_pad + 665 , y_pad + 75, 16)
    text_to_surface(screen, "Black Planet", "c" , y_pad + 25, 72)
    msg = "Play game"
    text_to_surface(screen, msg, x_pad + 320, y_pad + 185, 48)
    msg = "Options"
    text_to_surface(screen, msg, x_pad + 320, y_pad + 260, 48)
    msg = "Exit game"
    text_to_surface(screen, msg, x_pad + 320, y_pad + 335, 48)
    msg = msg_copyright
    text_to_surface(screen, msg, "c" , y_pad + 550, 16)
    msg = msg_license
    text_to_surface(screen, msg, "c" , y_pad + 575, 16)
    player[0].new_place(x_pad + 295, y_pad + 210 + choice * 75)
    selector.draw(screen)
    yellow = p.image.load('data/yellow_alt.png')
    screen.blit(yellow, (x_pad + 50, y_pad + 340))
    green = p.image.load('data/green_alt.png')
    screen.blit(green, (x_pad + 50, y_pad + 265))
    blue = p.image.load('data/blue_alt.png')
    screen.blit(blue, (x_pad + 50, y_pad + 190))
    red = p.image.load('data/red_alt.png')
    screen.blit(red, (x_pad + 50, y_pad + 115))
    p.draw.circle(screen,(0, 0, 0), (x_pad + 145, y_pad + 150), 40, 0)
    p.display.flip()

#End game animation.
def end_game():
    snd_message.play()
    screen.set_clip()
    bg.set_clip()
    screen.blit(bg, (0, 0))
    score_board()
    p.display.flip()
    colors=((255, 0, 0), (0, 255, 0), (0, 0, 255), (255, 255, 0), (255, 0, 255), (0, 255, 255))
    winner = [0]
    if player[0].lives == 0:
        winner = [1]
    for max_radius in range(0, int(scale * 900), int(scale * 50)):
        color = 0
        base = 0
        repeat = 1
        while repeat :
            base = base + 1
            if base == 6:
                base = 0
            color = base
            for radius in range(max_radius, 0, -int(scale * 50)):
                for i in winner:
                    p.draw.circle(screen, colors[color % 6], player[i].rect.center, radius, 2)
                color = color + 1
            playersprites.draw(screen)
            p.display.flip()
            clock.tick(8)
            if max_radius < scale * 850:
                repeat = 0
            for event in p.event.get():
                if event.type == QUIT:
                    sys.exit()
                if event.type == KEYDOWN:
                    if event.key == K_SPACE or event.key == K_ESCAPE:
                        repeat = 0
    snd_message.stop()

#Create scoresprites
def score_board():
    global scoresprites
    global score, shots
    for i in range(len(score) - 1, -1, -1):
        score[i].kill()
        del score[i]
    y = 20
    for j in [0, 1]:
        for i in range(player[j].lives):
            if j:
                x = width - 20 - i * 25
            else:
                x = 20 + i * 25
            score.append(Life(j))
            score[len(score) - 1].new_place(x, y)
    scoresprites = p.sprite.RenderUpdates(score)

#Toggle fullscreen
def init_display(mode = 0, width = 800, height = 600):
    if mode:
        return p.display.set_mode((width, height), FULLSCREEN)
    else:
        return p.display.set_mode((width, height))

#Start round
def round_start():
    global turn, shots_round, rotate_diff, velocity_diff, surf_highlight, bg_temp
    #Init the hightlight surface for the next round.
    surf_highlight = [p.Surface(screen.get_size(), SRCALPHA, 32).convert_alpha(), p.Surface(screen.get_size(), SRCALPHA, 32).convert_alpha()]
    surf_transition_new = p.Surface(screen.get_size(), SRCALPHA, 32).convert_alpha()
    rotate_diff = 0
    velocity_diff = 0
    shots_round = 0
    initplayers()
    createbg(5, 1000)
    #for the aiming sight to work ptoperly for the first shooter
    bg_temp = bg.copy()
    screen.set_clip()
    surf_transition_old = screen.copy()
    surf_transition_new.blit(bg, (0,0))
    score_board()
    scoresprites.draw(surf_transition_new)
    #transition effect
    for i in range(width - 1, 0, -int(scale * 30)):
		clock.tick(scale * 60)
		screen.blit(surf_transition_old, (i - width, 0))
		screen.blit(surf_transition_new, (i, 0))
		p.display.flip()
    screen.blit(surf_transition_new, (0, 0))
    p.display.flip()
    snd_enter.play()
    player[1 - turn].zoom_in()
    snd_enter.play()
    player[turn].zoom_in()
    p.event.clear()

#Read values from init file.
def read_init(param):
    config = confpar.ConfigParser()
    try:
        config.readfp(open('init.cfg'))
        result = config.getint('Options', param)
    except Exception:
        if param == 'fullscreen':
            write_init(param, 0)
            result = 0
        elif param == 'max_lives':
            write_init(param, 5)
            result = 5
        elif param == 'width':
            write_init(param, 800)
            result = 800
        elif param == 'height':
            write_init(param, 600)
            result = 600
        elif param =='show_planets':
            write_init(param, 1)
            result = 1
        elif param =='draw_traces':
            write_init(param, 1)
            result = 1
        elif param =='draw_forces':
            write_init(param, 1)
            result = 1
    return result

#Write values to init file
def write_init(param, value):
    config = confpar.ConfigParser()
    try:
        config.readfp(open('init.cfg'))
    except Exception:
        pass
    if config.has_section('Options') != 1:
        config.add_section('Options')
    config.set('Options', param, value)
    try:
        config.write(open('init.cfg', 'wb') )
    except Exception:
        pass

def resolution_init():
    global fullscreen, width, height, scale, x_off, y_off, screen, player, playersprites, bg_temp
    global bg, surf_planets, surf_planets_edge, surf_planets_hole, surf_stars, score_text, score_text_sprites, surf_highlight
    display_modes = p.display.list_modes()
    for i in range(0,len(display_modes), 1):
        if width == display_modes[i][0] and height == display_modes[i][1]:
            break
        elif i == len(display_modes)-1:
            fullscreen = 0
            width = 800
            height = 600
    screen = init_display(fullscreen, width, height)
 
    scale = min(width / 800.0, height / 600.0)
    x_off = int((width - scale * 800) / 2)
    y_off = int((height - scale * 600) / 2)
		
    bg = p.Surface(screen.get_size()).convert_alpha()
    bg_temp = p.Surface(screen.get_size()).convert_alpha()
    surf_planets = p.Surface(screen.get_size(), SRCALPHA, 32).convert_alpha()
    surf_planets_edge = p.Surface(screen.get_size(), SRCALPHA, 32).convert_alpha()
    surf_planets_hole = p.Surface(screen.get_size(), SRCALPHA, 32).convert_alpha()
    surf_highlight = [p.Surface(screen.get_size(), SRCALPHA, 32).convert_alpha(), p.Surface(screen.get_size(), SRCALPHA, 32).convert_alpha()]
    surf_stars = p.Surface(screen.get_size()).convert_alpha()
    for i in range(len(player) - 1, -1, -1):
        player[i].kill()
        del player[i]
    player = [Player(0), Player(1)]
    playersprites = p.sprite.RenderUpdates(player)
    score_text = Score()

def aftermath(cool_ratio, victim, shooter):
    screen.set_clip()
    round_ratio = int(cool_ratio)
    if victim != shooter and round_ratio > 1:
        msg = "x%i shot!" % (round_ratio)
        text_to_surface(screen, msg, "c", "c", 72)
        p.display.flip()
        time.sleep(1)
    if victim != shooter and cool_ratio == 0:
        msg = "One-shot!"
        text_to_surface(screen, msg, "c", "c", 72)
        p.display.flip()
        time.sleep(1)
    if victim == shooter:
        msg = "Darwin shot!"
        text_to_surface(screen, msg, "c", "c", 72)
        p.display.flip()
        time.sleep(1)
    screen.blit(bg, (0,0))
    scoresprites.draw(screen)
    playersprites.draw(screen)
    p.display.flip()

#Main program
draw_traces = read_init('draw_traces')
draw_forces = read_init('draw_forces')
show_planets = read_init('show_planets')
fullscreen = read_init('fullscreen')
max_lives = read_init('max_lives')
width = read_init('width')
height = read_init('height')
player = []
playersprites = []
score_text = []
fontface = "data/gravitat.ttf"
p.mixer.pre_init(44100, -16, 2, 1024)
p.init()
resolution_init()
missile = Missile()
trail = [Puff(12), Puff(10), Puff(8), Puff(6), Puff(4)]
hit = [Impact(0), Impact(1)]
planets = []
score = []
scoresprites = []
turn = 0
shots_round = 0
shots = 0
clock = p.time.Clock()
snd_enter = p.mixer.Sound("data/enter.wav")
snd_message = p.mixer.Sound("data/message.wav")
snd_damage = p.mixer.Sound("data/damage.wav")
snd_explosion = p.mixer.Sound("data/explosion.wav")
snd_fire = p.mixer.Sound("data/fire.wav")
p.mouse.set_visible(0)
p.key.set_repeat(500, 3)
p.display.set_caption('Black Planet')
game_start();
round_start();
rotate_diff = 0
velocity_diff = 0
run = 1
focus_anim = 1
#Main loop
while run:
    clock.tick(60)
    for event in p.event.get():
        if event.type == QUIT:
            sys.exit()
        if event.type == KEYDOWN:
            if event.key == K_LEFT:
                if event.key == K_LCTRL:
                    print "1"
                rotate(turn,0)
                rotate_diff=rotate_diff + 0.5
                if (rotate_diff == 360):
                    rotate_diff = 0
            if event.key == K_RIGHT:
                rotate(turn,1)
                rotate_diff=rotate_diff - 0.5
                if (rotate_diff == -360):
                    rotate_diff = 0
            if event.key == K_DOWN:
                if (player[turn].power > 60):
                    velocity_diff=velocity_diff - 1
                velocity(turn,0)
            if event.key == K_UP:
                if (player[turn].power < 100):
                    velocity_diff=velocity_diff + 1
                velocity(turn,1)
            if event.key == K_ESCAPE:
                game_start()
                round_start()
            if event.key == K_SPACE or event.key == K_RETURN:
                rotate_diff = 0
                velocity_diff = 0
                shooter = turn
                shots_round = shots_round + 1
                shots = shots + 1
                #Swap turn.
                turn = (turn + 1) % 2
                result = trace_torpedo(shooter)
                # Emp hits player.
                if result[0]==0 or result[0]==1:
                    # Trace player in the gravitational field.
                    result2 = trace_player(result[0], result[1], result[2])
                    screen.set_clip()
                    screen.blit(bg, (0, 0))
                    playersprites.draw(screen)
                    scoresprites.draw(screen)
                    p.display.flip()
                    # If player hits a planet, draw explosion.
                    if (result2[0] != 2 and result2[0] != 4):
                        explosion(result2[1], result2[2], 21, result[0])
                        # Take a life away from hitting player.
                        if shots_round <= 2:
                            aftermath(0, result[0], shooter)
                        aftermath(result[3], result[0], shooter)
                        player[result[0]].lives = player[result[0]].lives - 1
                        score_board()
                        time.sleep(1.5)
                    # If player hits other player.
                    if result2[0] == 4:
                        explosion(result2[1], result2[2], 21, 3)
                        if shots_round <= 2:
                            aftermath(0, result[0], shooter)
                        aftermath(result[3], result[0], shooter)
                        # Randomize turn.
                        turn = r.randint(0, 1)
                    # Player leaves the system.
                    if result2[0] == 2:
                        aftermath(result[3], result[0], shooter)
                        if shots_round <= 2:
                            aftermath(0, result[0], shooter)
                        player[result[0]].lives = player[result[0]].lives - 1
                        score_board()
                    # Any lives left?
                    if player[0].lives > 0 and player[1].lives > 0:
                        round_start()
                    else:
                        end_game()
                        initplayers()
                        game_start()
                        round_start()
                # Emp hits a planet.
                elif result[0]==3:
                    explosion(result[1], result[2], 5, 2)
                    time.sleep(0.9)
                focus_anim = 1

    p.event.clear()
    if focus_anim:
        focus()
        focus_anim=0
    draw_aim(rotate_diff, velocity_diff)
